﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Data.Entity;
using CodeFirstSample.PersonService;
using System.Xml.Linq;
using System.IO;

namespace CodeFirstSample
{
  class Program
  {
    static void Main(string[] args)
    {
      //QueryPerson();
      //InsertPerson();
      InsertPersonNoLastName();
      //InsertMultiplePersonNoLastName();

    }

    private static void QueryPerson()
    {
      //verify service is accessible
      var context = new PersonModelContext(new Uri("http://localhost:43447/DataService.svc"));
      try
      {
        var people = context.People.ToList();
        Console.WriteLine("Success: " + people.Count.ToString() + " retrieved");
      }
      catch (Exception ex)
      {
        Console.WriteLine("Query Failed");
        if (ex.InnerException != null)
          Console.WriteLine("Inner Exception: " + ex.InnerException.Message);
      }
      finally
      {
        FinishConsole();
      }
    }

    //verify that VALID insert succeeds
    private static void InsertPerson()
    {
      var person = new Person
      {
        FirstName = "Julie",
        LastName = "Lerman",
        IdentityCardNumber = "12345678",
      };
      var context = new PersonModelContext(new Uri("http://localhost:43447/DataService.svc"));

      context.AddToPeople(person);
      try
      {
        context.SaveChanges();
        Console.WriteLine("Successful Insert");
      }
      catch (Exception ex)
      {
        Console.WriteLine("Insert Failed");
        if (ex.InnerException != null)
          Console.WriteLine("Inner Exception: " + ex.InnerException.Message);
      }
      finally
      {
        FinishConsole();
      }
    }


    private static void InsertPersonNoLastName()
    {
      var person = new Person
      {
        FirstName = "Julie",
        IdentityCardNumber = "123456789ABCDE"
      };
      var context = new PersonModelContext(new Uri("http://localhost:43447/DataService.svc"));


      context.AddToPeople(person);

      try
      {
        context.SaveChanges();
        Console.WriteLine("Insert Succeeded -- but that means this little test failed!");
      }

      catch (Exception ex)
      {
        var sr = new StringReader(ex.InnerException.Message);
        XElement root = XElement.Load(sr);
        IEnumerable<XElement> message =
            from el in root.Elements()
            where el.Name.LocalName == "message"
            select el;
        foreach (XElement el in message)
          Console.WriteLine(el.Value);
      }
      finally {
        FinishConsole();
      }
    }

    private static void InsertMultiplePersonNoLastName()
    {
      var person = new Person
      {
        FirstName = "Julie",
        IdentityCardNumber = "123456789ABCDE"
      };
      var person2 = new Person
      {
        FirstName = "Rich",
        IdentityCardNumber = "987654321EDCBA"
      };
      var context = new PersonModelContext(new Uri("http://localhost:43447/DataService.svc"));


      context.AddToPeople(person);
      context.AddToPeople(person2);

      try
      {
        context.SaveChanges();
        Console.WriteLine("Insert Succeeded -- but that means this little test failed!");

      }

      catch (Exception ex)
      {
        var sr = new StringReader(ex.InnerException.Message);
        XElement root = XElement.Load(sr);
        IEnumerable<XElement> message =
            from el in root.Elements()
            where el.Name.LocalName == "message"
            select el;
        foreach (XElement el in message)
          Console.WriteLine(el.Value);

        Console.ReadKey();
      }
      finally
      {
        FinishConsole();
      }
    }

    private static void FinishConsole()
    {
      Console.WriteLine("Press any key to end ...");
      Console.ReadKey();
    }
  }
}
