﻿using System.ComponentModel.DataAnnotations;
using System.Data.Entity;
using System.Data.Entity.ModelConfiguration;
using Model;

namespace DataAccess
{
  public class BreakAwayContextFluent : DbContext
  {
    public DbSet<Destination> Destinations { get; set; }
    public DbSet<Lodging> Lodgings { get; set; }
    public DbSet<Trip> Trips { get; set; }
    public DbSet<Person> People { get; set; }
    public DbSet<PersonPhoto> PersonPhotos { get; set; }

    protected override void OnModelCreating(DbModelBuilder modelBuilder)
    {

      modelBuilder.Entity<Lodging>()
    .Map(m =>
    {
      m.ToTable("Lodging");
    })
    .Map<Resort>(m =>
    {
      m.ToTable("Resorts");
      m.MapInheritedProperties();
    });
	 

     
      //  modelBuilder.Entity<Resort>().Map(m=>
      // {
      //  m.ToTable("Resorts");
      //  m.MapInheritedProperties();
      //});
      modelBuilder.Configurations.Add(new DestinationConfiguration());
     // modelBuilder.Configurations.Add(new LodgingConfiguration());
      modelBuilder.Configurations.Add(new PersonConfiguration());
      modelBuilder.Configurations.Add(new TripConfiguration());
      modelBuilder.Configurations.Add(new AddressConfiguration());
      modelBuilder.Configurations.Add(new PersonPhotoConfiguration());
      modelBuilder.Entity<Person>().Property(p => p.Address.StreetAddress).HasColumnName("StreetAddress");
      modelBuilder.ComplexType<Address>().Property(p => p.StreetAddress).HasColumnName("StreetAddress");
      modelBuilder.ComplexType<Address>();
      modelBuilder.Entity<InternetSpecial>().HasRequired(s => s.Accommodation).WithMany(l => l.InternetSpecials).
        HasForeignKey(s => s.AccommodationId);
      modelBuilder.Entity<Destination>().HasMany(d => d.Lodgings).WithRequired(l => l.Destination);
      modelBuilder.Entity<Person>().ToTable("People");
      modelBuilder.Entity<PersonPhoto>().ToTable("People");
      // modelBuilder.Entity<Resort>().ToTable("Resorts");


      //modelBuilder.Entity<Lodging>().Map(m =>
      //{
      //  m.ToTable("Lodging");
      //}).Map<Resort>(m =>
      //{
      //  m.ToTable("Resorts");
      //});




      ////modelBuilder.Entity<Destination>().Property(d => d.Name).IsRequired();
      //modelBuilder.Entity<Destination>().Property(d => d.Description).HasMaxLength(500);
      //modelBuilder.Entity<Destination>().Property(d => d.Photo).HasColumnType("image");
      //modelBuilder.Entity<Trip>().HasKey(t => t.Identifier).Property(t => t.Identifier).HasDatabaseGeneratedOption(
      //  DatabaseGeneratedOption.Identity);
      //modelBuilder.Entity<Person>().HasKey(p => p.SocialSecurityNumber);
      //modelBuilder.Entity<Person>().Property(p => p.SocialSecurityNumber).HasDatabaseGeneratedOption(
      //  DatabaseGeneratedOption.None);
      //modelBuilder.Entity<Person>().Property(p => p.RowVersion).IsRowVersion();
      //modelBuilder.Entity<Lodging>().Property(l => l.Name).IsRequired().HasMaxLength(200);

      //modelBuilder.Entity<Lodging>()
      //    .Property(l => l.Owner)
      //    .IsRequired()
      //    .HasMaxLength(200);

      //modelBuilder.Entity<Destination>()
      //    .Property(d => d.Description)
      //    .HasColumnType("text");

      //modelBuilder.Entity<Destination>()
      //    .Property(d => d.Country)
      //    .IsConcurrencyToken();


    }
  }
  public class AddressConfiguration : ComplexTypeConfiguration<Address>
  {
    public AddressConfiguration()
    {
      Property(a => a.State).HasMaxLength(30);
      Property(a => a.StreetAddress).HasColumnName("StreetAddress");
    }
  }

  public class PersonConfiguration : EntityTypeConfiguration<Person>
  {
    public PersonConfiguration()
    {
      /*
        HasKey(p => p.SocialSecurityNumber);
        Property(p => p.SocialSecurityNumber).HasDatabaseGeneratedOption(
        DatabaseGeneratedOption.None);
        Property(p => p.RowVersion).IsRowVersion();
       */
      Property(p => p.SocialSecurityNumber).IsConcurrencyToken();
      Property(p => p.Address.StreetAddress).HasColumnName("StreetAddress");
      // HasRequired(p => p.Photo).WithRequiredPrincipal(pp=>pp.PhotoOf).WillCascadeOnDelete(true);
      ToTable("People"); //added along when mapping multiple entites to one table
    }
  }
  public class TripConfiguration : EntityTypeConfiguration<Trip>
  {
    public TripConfiguration()
    {
      HasKey(t => t.Identifier).Property(t => t.Identifier).HasDatabaseGeneratedOption(
          DatabaseGeneratedOption.Identity);
      Property(p => p.RowVersion).IsRowVersion();
    }
  }


  public class PersonPhotoConfiguration : EntityTypeConfiguration<PersonPhoto>
  {
    public PersonPhotoConfiguration()
    {
      this.HasKey(p => p.PersonId);
      HasRequired(p => p.PhotoOf)
       .WithRequiredDependent(p => p.Photo);
      ToTable("People");

    }
  }

  public class DestinationConfiguration : EntityTypeConfiguration<Destination>
  {
    public DestinationConfiguration()
    {
      this.Property(d => d.Name).IsRequired().HasColumnName("LocationName");
      Property(d => d.DestinationId).HasColumnName("LocationID");
      Property(d => d.Description).HasMaxLength(500);
      Property(d => d.Photo).HasColumnType("image");
      Ignore(d => d.TodayForecast);
      // ToTable("Locations");

      Map(m =>
          {
            m.Properties(d => new { d.Name, d.Country, d.Description });
            m.ToTable("Locations");
          });
      Map(m =>
          {
            m.Properties(d => new { d.Photo });
            m.ToTable("LocationPhotos");

          });
    }
  }
  public class LodgingConfiguration : EntityTypeConfiguration<Lodging>
  {
    public LodgingConfiguration()
    {
      //Map(m =>
      //{
      //  m.ToTable("Lodgings");
      //})
      //   .Map<Resort>(m =>
      //   {
      //     m.ToTable("Resorts");
      //     m.MapInheritedProperties();
      //   });
      Property(l => l.Name).IsRequired().HasMaxLength(200);
      Property(l => l.Owner).IsUnicode(false);
      Property(l => l.MilesFromNearestAirport).HasPrecision(8, 1);
      HasOptional(l => l.PrimaryContact).WithMany(p => p.PrimaryContactFor);
      HasOptional(l => l.SecondaryContact).WithMany(p => p.SecondaryContactFor);
     
      
      //   HasRequired(l => l.Destination).WithMany(d => d.Lodgings).WillCascadeOnDelete(false);

      //Map(m =>
      //    {
      //        m.ToTable("Lodging");
      //        m.Requires("LodgingType").HasValue("Standard");
      //    })
      //.Map<Resort>(m =>
      //    {
      //        m.Requires("LodgingType").HasValue("Resort");
      //    });

      //Map(m =>
      //{
      //  m.ToTable("Lodging");
      //  m.Requires("IsResort").HasValue(false);
      //})
      //.Map<Resort>(m =>
      //{
      //  m.Requires("IsResort").HasValue(true);
      //});


    }


  }
}



